#ifndef SCENETREE_H
#define SCENETREE_H

#include <vector>
#include <string>
#include "scene.h"
#include "light.h"
#include "resourcemanager.h"

/**
 * @brief The SceneNode class represents a class of the game
 * that will be updated or used for rendering every frame
 */
class SceneNode
{
public:
    SceneNode* m_parent;
    virtual void update() = 0;
};

/**
 * @brief The ContainerNode class represents a node which main use is to contain a collection of nodes
 */
class ContainerNode : public SceneNode
{
public:
    virtual void update();
    void addChild(const std::string &childName);
    void removeChild(const std::string &childName);
protected:
    std::vector<SceneNode*> m_children;
};

/**
 * @brief The CameraNode class is a scene node that can be used by the renderer
 */
class CameraNode : public Camera, SceneNode
{

};

class SceneTree : public Scene
{
public:
    SceneTree(std::string name);
    ~SceneTree();

    virtual SceneIterator<Light*>* getLights();
    virtual SceneIterator<GeometryNode*>* getGeometry();

    void update();

    Texture* getSkybox() {return m_skybox;}
    void setSkybox(Texture* skybox) {m_skybox = skybox;}

    void setMainCamera(const std::string &name) {m_camera = RESOURCE_GET(CameraNode, name);}
    void addRootObject(const std::string &name) {m_root.addChild(name);}

private:
    std::string m_name;
    ContainerNode m_root;
    Texture* m_skybox;
};

// Additionnal Nodes :



#endif // SCENETREE_H
